package xmlchecker;

import xmlchecker.XMLCheckerFactory;
import xmlchecker.XMLChecker;
import xmlchecker.XMLTreeFrame;
import xmlchecker.FontScaler;

import java.io.File;
import java.io.FileOutputStream;
import java.io.PrintWriter;
import java.io.StringReader;
import java.io.BufferedReader;
import java.io.IOException;

import java.net.URI;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JSlider;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;
import javax.swing.JOptionPane;
import javax.swing.BorderFactory;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;

/**
  Hauptfenster der grafischen Oberflche
*/
public class XMLCheckerGUIMain extends JPanel{
    private JTextArea txtMessages;
    private File checkedFile;
    private JTextField txtFileXML;
    private JButton btnFileXML, btnCheck, btnFileLog;
    private JCheckBox chkVal, chkSchema, chkNS, chkTree;
    private JSlider sldScale;
    private XMLTreeFrame treeFrame;
    private JFileChooser chooserXML, chooserLog;
    private XMLCheckerFactory checkerFactory;
    private FontScaler scaler;

/**
  StandardKonstruktor
*/
  public XMLCheckerGUIMain(){
//Objekte instanziieren
    this.checkerFactory = XMLCheckerFactory.newInstance();
    this.chooserXML = new JFileChooser();
    this.chooserXML.setDialogTitle("XML Dokument auswhlen");
    chooserXML.addChoosableFileFilter(new FileFilter(){
      public boolean accept(File f){
        return f.getName().toLowerCase().endsWith(".xml") || f.isDirectory();
      }
      public String getDescription(){
        return "xml Dokument";
      }
    });
    this.chooserLog = new JFileChooser();
    this.chooserLog.setDialogTitle("Meldungen speichern");

    this.scaler = new FontScaler();
    this.treeFrame = new XMLTreeFrame(scaler);

    this.txtMessages = new JTextArea(3,40);
    this.txtMessages.setFont(new Font("dialog", Font.PLAIN, 12));
    this.txtFileXML = new JTextField(30);
    this.btnFileXML = new JButton("...");
    this.btnFileXML.setToolTipText("XML Dokument auswhlen");
    this.btnCheck = new JButton("parse!");
    this.btnFileLog = new JButton("...");
    this.btnFileLog.setEnabled(false);
    this.btnFileLog.setToolTipText("Meldungen speichern");
    this.chkVal = new JCheckBox("validieren", true);
    this.chkSchema = new JCheckBox("Schema verwenden", true);
    this.chkNS = new JCheckBox("Namensraum erkennen", true);
    this.chkTree = new JCheckBox("Baum erstellen", true);
    this.sldScale = new JSlider(8, 20, 10);
    this.sldScale.setSnapToTicks(true);
    this.sldScale.setToolTipText("Schriftgre 100%");

//Objekte positionieren
    JPanel messagePanel = new JPanel(new BorderLayout());
    messagePanel.add(new JScrollPane(this.txtMessages), BorderLayout.CENTER);
    messagePanel.add(this.btnFileLog, BorderLayout.EAST);
    messagePanel.setBorder(BorderFactory.createCompoundBorder(
      BorderFactory.createTitledBorder("Meldungen"),
      BorderFactory.createEmptyBorder(4, 4, 4, 4)
    ));
    JPanel filePanel = new JPanel(new BorderLayout());
    filePanel.add(this.txtFileXML, BorderLayout.CENTER);
    filePanel.add(this.btnFileXML, BorderLayout.EAST);
    filePanel.add(this.btnCheck, BorderLayout.SOUTH);
    filePanel.setBorder(BorderFactory.createCompoundBorder(
      BorderFactory.createTitledBorder("XML-Dokument"),
      BorderFactory.createEmptyBorder(4, 4, 4, 4)
    ));
    JPanel configPanel = new JPanel(new GridLayout(2, 2));
    configPanel.add(this.chkVal);
    configPanel.add(this.chkSchema);
    configPanel.add(this.chkNS);
    configPanel.add(this.chkTree);
    configPanel.setBorder(BorderFactory.createCompoundBorder(
      BorderFactory.createTitledBorder("Parser-Konfiguration"),
      BorderFactory.createEmptyBorder(4, 4, 4, 4)
    ));
    JPanel northPanel = new JPanel(new GridLayout(1, 2));
    northPanel.add(filePanel);
    northPanel.add(configPanel);

    this.setLayout(new BorderLayout());
    this.add(northPanel, BorderLayout.NORTH);
    this.add(messagePanel, BorderLayout.CENTER);
    this.add(sldScale, BorderLayout.SOUTH);

    this.setPreferredSize(new Dimension(720, 360));

//Fontskalierung
    this.scaler.add(txtMessages);
    this.scaler.add(txtFileXML);
    this.scaler.add(btnCheck);
    this.scaler.add(chkVal);
    this.scaler.add(chkSchema);
    this.scaler.add(chkNS);
    this.scaler.add(chkTree);

//Ereignisbehandlung
    chkVal.addItemListener(new ItemListener(){
      public void itemStateChanged(ItemEvent ie){
        XMLCheckerGUIMain.this.chkSchema.setEnabled(ie.getStateChange() == ItemEvent.SELECTED);
      }
    });
    chkTree.addItemListener(new ItemListener(){
      public void itemStateChanged(ItemEvent ie){
        if(XMLCheckerGUIMain.this.treeFrame.isVisible()){
          XMLCheckerGUIMain.this.treeFrame.setVisible(false);
	    }
      }
    });
    sldScale.addChangeListener(new ChangeListener(){
      public void stateChanged(ChangeEvent ce){
        int val = XMLCheckerGUIMain.this.sldScale.getValue();
        XMLCheckerGUIMain.this.scaler.scale(val / 10.0f);
        XMLCheckerGUIMain.this.treeFrame.repaint();
        XMLCheckerGUIMain.this.sldScale.setToolTipText("Schriftgre " + (val * 10) + "%");
      }
    });

    btnFileXML.addActionListener(new ActionListener(){
      public void actionPerformed(ActionEvent ae){
        XMLCheckerGUIMain.this.fileXML();
      }
    });
    btnCheck.addActionListener(new ActionListener(){
      public void actionPerformed(ActionEvent ae){
        XMLCheckerGUIMain.this.check();
      }
    });
    btnFileLog.addActionListener(new ActionListener(){
      public void actionPerformed(ActionEvent ae){
        XMLCheckerGUIMain.this.fileLog();
      }
    });
  }

/**
  XML-Dokument auswhlen
*/
  public void fileXML(){
    this.chooserXML.setCurrentDirectory(new File(System.getProperty("user.dir")));
    String filename = this.txtFileXML.getText();
    if(filename.length() > 0){
      try{
        File fileentry = new File(filename);
        if(fileentry.isDirectory()){
          this.chooserXML.setCurrentDirectory(fileentry);
        } else {
          this.chooserXML.setSelectedFile(fileentry);
        }
      }catch(Exception ex){
      }
    }
    if(chooserXML.showOpenDialog(this) == JFileChooser.APPROVE_OPTION){
      this.txtFileXML.setText(this.chooserXML.getSelectedFile().toString());
    }
  }
/**
  Ausgewhltes XML-Dokument prfen
*/
  public void check(){
    XMLChecker checker = this.checkerFactory.newXMLChecker();
    checker.setNamespaceAware(this.chkNS.isSelected());
    checker.setValidating(this.chkVal.isSelected());
    if(this.chkVal.isSelected()){
      checker.setUsingSchema(this.chkSchema.isSelected());
    } else {
      checker.setUsingSchema(false);
    }
    checker.setCreatingXMLTree(this.chkTree.isSelected());
    this.btnFileLog.setEnabled(false);
    this.checkedFile = new File(this.txtFileXML.getText());
    checker.check(this.checkedFile.toURI().toString());
    this.txtMessages.setText(checker.getMessages());
    this.btnFileLog.setEnabled(true);
    if(this.chkTree.isSelected()){
      this.treeFrame.setTitle(this.txtFileXML.getText());
      this.treeFrame.setRoot(checker.getTreeRoot());
      if(!this.treeFrame.isDisplayable()){
        this.treeFrame.setLocation(this.getLocationOnScreen());
      }
      this.treeFrame.setVisible(true);
    }
  }
/**
  Inhalt des Textfeldes Meldungen speichern
*/
  public void fileLog(){
    this.chooserLog.setSelectedFile(new File(this.checkedFile.getName() + "-err.txt"));
    if(this.chooserLog.showSaveDialog(this) == JFileChooser.APPROVE_OPTION){
      File log = this.chooserLog.getSelectedFile();
      if(log.exists() && log.isFile()){
        if(JOptionPane.showConfirmDialog(this.txtMessages,
                                         "Datei existiert bereits! berschreiben?",
                                         "XML-Checker", JOptionPane.YES_NO_OPTION,
                                         JOptionPane.WARNING_MESSAGE)!=JOptionPane.YES_OPTION){
          return;
        }
      }
      try{
        BufferedReader brmsg = new BufferedReader(new StringReader(this.txtMessages.getText()));
        PrintWriter pwlog = new PrintWriter(new FileOutputStream(log));
        String line = brmsg.readLine();
        while(line != null){
          pwlog.println(line);
          line = brmsg.readLine();
        }
        brmsg.close();
        pwlog.close();
      }catch(IOException ioex){
        JOptionPane.showMessageDialog(this.txtMessages,
                                     "Fehler: Datei nicht geschrieben",
                                     "XML-Checker",
                                     JOptionPane.ERROR_MESSAGE);
      }
    }
  }
}