package xmlchecker;

import xmlchecker.XMLTreeNode;
import xmlchecker.XMLTreeRenderer;
import xmlchecker.FontScaler;

import java.util.Enumeration;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.JButton;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;

/**
  Anzeige des XML-Dokumentes als Baum
*/
public class XMLTreeFrame extends JFrame{
  private DefaultTreeModel treeModel;
  private JTree xmlTree;
  private JButton btnExpandAll, btnCollapseAll;

/**
  Konstruktor
  @param   scaler   Skalierung fr die Texte im Baum
*/
  public XMLTreeFrame(FontScaler scaler){
    super("");
//Objekte instanziieren
    this.treeModel = new DefaultTreeModel(new XMLTreeNode());
    this.xmlTree = new JTree(this.treeModel);
    xmlTree.setCellRenderer(new XMLTreeRenderer(scaler));
    this.btnExpandAll = new JButton("Alle Knoten auf");
    this.btnCollapseAll = new JButton("Alle Knoten zu");

//Objekte positionieren
    JPanel southPanel = new JPanel(new GridLayout(1, 2));
    southPanel.add(this.btnExpandAll);
    southPanel.add(this.btnCollapseAll);

    this.getContentPane().add(new JScrollPane(this.xmlTree), BorderLayout.CENTER);
    this.getContentPane().add(southPanel, BorderLayout.SOUTH);
    this.setSize(320, 480);

//Ereignisbehandlung
    btnExpandAll.addActionListener(new ActionListener(){
      public void actionPerformed(ActionEvent ae){
        XMLTreeFrame.this.expandAll();
      }
    });
    btnCollapseAll.addActionListener(new ActionListener(){
      public void actionPerformed(ActionEvent ae){
        XMLTreeFrame.this.collapseAll();
      }
    });

    this.setDefaultCloseOperation(JFrame.HIDE_ON_CLOSE);
  }

/**
  Wurzelknoten setzen
  @param   root   der neue Wurzelknoten des Baums
*/
  public void setRoot(XMLTreeNode root){
    this.treeModel.setRoot(root);
  }

/**
  alle Knoten ausklappen
*/
  public void expandAll(){
    this.expandChilds((XMLTreeNode)this.xmlTree.getModel().getRoot());
  }
/**
  alle Kindknoten ausklappen
  @param   node   Knoten, unter dem der Teilbaum aufgeklappt werden soll
*/
  public void expandChilds(XMLTreeNode node){
    this.xmlTree.expandPath(new TreePath(node.getPath()));
    Enumeration childs = node.children();
    while(childs.hasMoreElements()){
      XMLTreeNode child = (XMLTreeNode)childs.nextElement();
      this.expandChilds(child);
    }
  }
/**
  alle Knoten zuklappen
*/
  public void collapseAll(){
    this.collapseChilds((XMLTreeNode)this.xmlTree.getModel().getRoot());
  }
/**
  alle Kindknoten zuklappen
  @param   node   Knoten, unter dem der Teilbaum zugeklappt werden soll
*/
  public void collapseChilds(XMLTreeNode node){
    Enumeration childs = node.children();
    while(childs.hasMoreElements()){
      XMLTreeNode child = (XMLTreeNode)childs.nextElement();
      this.collapseChilds(child);
    }
    this.xmlTree.collapsePath(new TreePath(node.getPath()));
  }
}