package xmlchecker.sax;

import xmlchecker.XMLChecker;
import xmlchecker.XMLTreeNode;

import javax.xml.parsers.SAXParserFactory;
import org.xml.sax.XMLReader;
import org.xml.sax.SAXException;

public class SAXChecker extends XMLChecker{
  public static final String NAMESPACES_FEATURE_ID = "http://xml.org/sax/features/namespaces";
  public static final String VALIDATION_FEATURE_ID = "http://xml.org/sax/features/validation";
  public static final String SCHEMA_VALIDATION_FEATURE_ID = "http://apache.org/xml/features/validation/schema";

  private XMLTreeNode treeRoot;
  private StringBuffer messages;
  private SAXParserFactory parserFactory = null;

  public SAXChecker(){
    treeRoot = null;
    messages = new StringBuffer();
    parserFactory = SAXParserFactory.newInstance();
  }

  public void check(String docuri){
    try {
      XMLReader reader = parserFactory.newSAXParser().getXMLReader();
      reader.setFeature(NAMESPACES_FEATURE_ID, this.isNamespaceAware());
      reader.setFeature(VALIDATION_FEATURE_ID, this.isValidating());
      reader.setFeature(SCHEMA_VALIDATION_FEATURE_ID, this.isUsingSchema());

      reader.setErrorHandler(new SAXCheckerErrorHandler(messages));
      if(this.isCreatingXMLTree()){
        treeRoot = new XMLTreeNode();
        reader.setContentHandler(new SAXCheckerContentHandler(treeRoot, this.isNamespaceAware()));
      }

      try {
        reader.parse(docuri);
        messages.append('\n' + docuri + " geparst!\n");
      } catch (Exception ex) {
        messages.append('\n' + "Parse-Fehler! " + ex.getMessage( ) + '\n');
      }
    } catch(Exception ex ) {
      messages.append("Keinen SAX-Parser erzeugt! \n" + ex.getMessage( ) + '\n');
    }
  }

  public XMLTreeNode getTreeRoot(){
    return this.treeRoot;
  }
  public String getMessages(){
    return new String(this.messages);
  }
}